@@ -11,7 +11,7 @@ use App::Sqitch::X qw(hurl);
 use Locale::TextDomain qw(App-Sqitch);
 use namespace::autoclean;
 
-our $VERSION = '0.9992';
+our $VERSION = '0.9993';
 
 requires 'dbh';
 requires 'sqitch';
@@ -69,12 +69,12 @@ sub _register_release {
     return $self;
 }
 
+sub _version_query { 'SELECT MAX(version) FROM releases' }
+
 sub registry_version {
     my $self = shift;
     try {
-        sprintf '%.1f', $self->dbh->selectcol_arrayref(
-            'SELECT MAX(version) FROM releases'
-        )->[0];
+        $self->dbh->selectcol_arrayref($self->_version_query)->[0];
     } catch {
         return 0 if $self->_no_table_error;
         die $_;
